@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2018, 2019, 2020, 2021 CERN.
+# Copyright (C) 2018, 2019, 2020, 2021, 2022 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -10,7 +10,6 @@
 
 import os
 from distutils.util import strtobool
-from enum import Enum
 
 from reana_commons.config import REANA_INFRASTRUCTURE_COMPONENTS_HOSTNAMES
 
@@ -64,18 +63,6 @@ SQLALCHEMY_POOL_SIZE = int(float(os.getenv("SQLALCHEMY_POOL_SIZE", 5)))
 SQLALCHEMY_POOL_TIMEOUT = int(float(os.getenv("SQLALCHEMY_POOL_TIMEOUT", 30)))
 """How many seconds to wait when retrieving a new connection from the pool?"""
 
-
-class QuotaResourceType(str, Enum):
-    """Possible quota policies.
-
-    Example:
-        QuotaPolicy.cpu == "cpu"  # True
-    """
-
-    cpu = "cpu"
-    disk = "disk"
-
-
 DEFAULT_QUOTA_RESOURCES = {
     "cpu": "processing time",
     "disk": "shared storage",
@@ -94,7 +81,10 @@ WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY = policies.split(",") if policies else
 """What quota types to update, if not specified all quotas will be calculated, if empty no quotas will be updated."""
 
 
-CRONJOB_DISK_QUOTA_UPDATE_POLICY = strtobool(
-    os.getenv("REANA_CRONJOB_DISK_QUOTA_UPDATE_POLICY", "false")
+PERIODIC_RESOURCE_QUOTA_UPDATE_POLICY = strtobool(
+    os.getenv("REANA_PERIODIC_RESOURCE_QUOTA_UPDATE_POLICY", "false")
 )
-"""Whether to run the cronjob disk quota updater."""
+"""Whether to run the periodic (cronjob) resource quota updater."""
+
+LIMIT_RESTARTS = 9
+"""Maximum number of times a workflow can be restarted."""
