@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2020, 2021 CERN.
+# Copyright (C) 2020, 2021, 2022 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -18,7 +18,7 @@ import pytest
 from reana_db.models import Resource, RunStatus, User, Workflow
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="package")
 def db():
     """Initialize database fixture."""
     from reana_db.database import init_db
@@ -40,7 +40,7 @@ def new_user(session, db):
 
 @pytest.fixture
 def run_workflow(session, new_user):
-    """Mocked workflow run factory."""
+    """Mock workflow run factory."""
 
     def _run_workflow(time_elapsed_seconds=0.5, finish=True):
         """Mock a workflow run."""
