@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2020, 2021 CERN.
+# Copyright (C) 2020, 2021, 2022 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -13,7 +13,6 @@ from uuid import uuid4
 import pytest
 import mock
 
-from reana_db.config import QuotaResourceType
 from reana_db.models import (
     ALLOWED_WORKFLOW_STATUS_TRANSITIONS,
     AuditLogAction,
@@ -25,8 +24,14 @@ from reana_db.models import (
     Workflow,
     WorkflowResource,
     RunStatus,
+    WorkspaceRetentionRule,
+    WorkspaceRetentionRuleStatus,
+)
+from reana_db.utils import (
+    get_default_quota_resource,
+    update_users_cpu_quota,
+    update_workspace_retention_rules,
 )
-from reana_db.utils import get_default_quota_resource
 
 
 def test_workflow_run_number_assignment(db, session, new_user):
@@ -83,6 +88,62 @@ def test_workflow_run_number_assignment(db, session, new_user):
     assert first_workflow_second_restart.run_number == 1.2
 
 
+def test_workflow_retention_rules(db, session, new_user):
+    """Test workflow retention rules."""
+
+    def add_workflow(name, restart=False):
+        workflow = Workflow(
+            id_=str(uuid4()),
+            name=name,
+            owner_id=new_user.id_,
+            reana_specification=[],
+            type_="serial",
+            logs="",
+            restart=restart,
+        )
+        session.add(workflow)
+        session.commit()
+        return workflow
+
+    def rule_status(workflow):
+        return workflow.retention_rules.first().status
+
+    retention_rules = [{"workspace_files": "**/*.root", "retention_days": 1}]
+    workflow = add_workflow("workflow")
+    workflow_restart = add_workflow("workflow", restart=True)
+    workflow_another_name = add_workflow("another_name")
+
+    workflow.set_workspace_retention_rules(retention_rules)
+    workflow_restart.set_workspace_retention_rules(retention_rules)
+    workflow_another_name.set_workspace_retention_rules(retention_rules)
+    assert rule_status(workflow) == WorkspaceRetentionRuleStatus.created
+
+    workflow.activate_workspace_retention_rules()
+    assert rule_status(workflow) == WorkspaceRetentionRuleStatus.active
+
+    workflow_restart.inactivate_workspace_retention_rules()
+    assert rule_status(workflow) == WorkspaceRetentionRuleStatus.inactive
+    assert rule_status(workflow_restart) == WorkspaceRetentionRuleStatus.inactive
+    assert rule_status(workflow_another_name) == WorkspaceRetentionRuleStatus.created
+
+    r = workflow_restart.retention_rules.first()
+    r.status = WorkspaceRetentionRuleStatus.pending
+    session.add(r)
+    session.commit()
+    assert workflow.workspace_has_pending_retention_rules()
+    assert workflow_restart.workspace_has_pending_retention_rules()
+    assert not workflow_another_name.workspace_has_pending_retention_rules()
+
+    rules = workflow.retention_rules.all()
+    with pytest.raises(Exception) as e:
+        update_workspace_retention_rules(rules, WorkspaceRetentionRuleStatus.created)
+    assert "Cannot transition workspace retention rule" in e.value.args[0]
+
+    with pytest.raises(Exception) as e:
+        workflow.set_workspace_retention_rules(retention_rules)
+    assert "duplicate key value violates unique constraint" in e.value.args[0]
+
+
 @mock.patch(
     "reana_commons.utils.get_disk_usage", return_value=[{"size": {"raw": "128"}}]
 )
@@ -94,7 +155,7 @@ def test_workflow_run_number_assignment(db, session, new_user):
         (RunStatus.created, RunStatus.stopped, False),
         (RunStatus.created, RunStatus.running, False),
         (RunStatus.created, RunStatus.pending, False),
-        (RunStatus.queued, RunStatus.failed, False),
+        (RunStatus.queued, RunStatus.failed, True),
         (RunStatus.queued, RunStatus.finished, False),
         (RunStatus.queued, RunStatus.stopped, False),
         (RunStatus.queued, RunStatus.running, False),
@@ -211,7 +272,7 @@ def test_access_token(db, session, new_user):
 )
 @mock.patch(
     "reana_db.models.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
-    [QuotaResourceType.cpu.value, QuotaResourceType.disk.value],
+    [ResourceType.cpu.name, ResourceType.disk.name],
 )
 def test_workflow_cpu_quota_usage_update(db, session, run_workflow):
     """Test quota usage update once workflow is finished/stopped/failed."""
@@ -228,29 +289,92 @@ def test_workflow_cpu_quota_usage_update(db, session, run_workflow):
     assert cpu_milliseconds >= time_elapsed_seconds * 1000
 
 
-@mock.patch(
-    "reana_db.models.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
-    [QuotaResourceType.cpu.value, QuotaResourceType.disk.value],
+@pytest.mark.parametrize(
+    "workflow_termination_quota_update_policy",
+    [
+        ([ResourceType.cpu.name, ResourceType.disk.name],),
+        ([ResourceType.cpu.name]),
+        ([ResourceType.disk.name]),
+        ([]),
+    ],
 )
-@mock.patch(
-    "reana_db.utils.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
-    [QuotaResourceType.cpu.value, QuotaResourceType.disk.value],
-)
-@mock.patch(
-    "reana_commons.utils.get_disk_usage", return_value=[{"size": {"raw": "128"}}]
-)
-def test_user_cpu_usage(db, session, new_user, run_workflow):
+@mock.patch("reana_db.utils.get_disk_usage", return_value=[{"size": {"raw": "128"}}])
+def test_workflow_termination_user_quota_usage(
+    db, session, new_user, run_workflow, workflow_termination_quota_update_policy
+):
     """Test aggregated CPU usage per user."""
-    time_elapsed_seconds = 0.5
-    num_workflows = 2
-    for n in range(num_workflows):
-        run_workflow(time_elapsed_seconds=time_elapsed_seconds)
+    with mock.patch(
+        "reana_db.models.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
+        workflow_termination_quota_update_policy,
+    ), mock.patch(
+        "reana_db.utils.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
+        workflow_termination_quota_update_policy,
+    ):
 
-    assert (
-        new_user.get_quota_usage()["cpu"]["usage"]["raw"]
-        >= num_workflows * time_elapsed_seconds * 1000
-    )
-    assert new_user.get_quota_usage()["disk"]["usage"]["raw"] == 128
+        time_elapsed_seconds = 0.5
+        num_workflows = 2
+        for n in range(num_workflows):
+            run_workflow(time_elapsed_seconds=time_elapsed_seconds)
+
+        quota_usage = new_user.get_quota_usage()
+
+        if ResourceType.cpu.name in workflow_termination_quota_update_policy:
+            assert (
+                quota_usage["cpu"]["usage"]["raw"]
+                >= num_workflows * time_elapsed_seconds * 1000
+            )
+        else:
+            quota_usage["cpu"]["usage"]["raw"] == 0
+
+        if ResourceType.disk.name in workflow_termination_quota_update_policy:
+            assert quota_usage["disk"]["usage"]["raw"] == 128
+        else:
+            quota_usage["disk"]["usage"]["raw"] == 0
+
+
+@pytest.mark.parametrize(
+    "workflow_termination_quota_update_policy, periodic_update",
+    [
+        ([], True),
+        ([], False),
+        ([ResourceType.cpu.name], False),
+        ([ResourceType.cpu.name], True),
+    ],
+)
+def test_all_users_cpu_quota_usage_update(
+    db,
+    session,
+    new_user,
+    run_workflow,
+    workflow_termination_quota_update_policy,
+    periodic_update,
+):
+    """Test CPU periodic update cronjob functionality."""
+    with mock.patch(
+        "reana_db.models.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
+        workflow_termination_quota_update_policy,
+    ), mock.patch(
+        "reana_db.utils.PERIODIC_RESOURCE_QUOTA_UPDATE_POLICY",
+        periodic_update,
+    ):
+        time_elapsed_seconds = 0.5
+        num_workflows = 2
+        for n in range(num_workflows):
+            run_workflow(time_elapsed_seconds=time_elapsed_seconds)
+
+        quota_cpu_usage = new_user.get_quota_usage()["cpu"]["usage"]["raw"]
+        if workflow_termination_quota_update_policy:
+            assert quota_cpu_usage >= num_workflows * time_elapsed_seconds * 1000
+        else:
+            assert quota_cpu_usage == 0
+
+        update_users_cpu_quota()
+
+        quota_cpu_usage = new_user.get_quota_usage()["cpu"]["usage"]["raw"]
+        if workflow_termination_quota_update_policy or periodic_update:
+            assert quota_cpu_usage >= num_workflows * time_elapsed_seconds * 1000
+        else:
+            assert quota_cpu_usage == 0
 
 
 @pytest.mark.parametrize(
@@ -271,10 +395,10 @@ def test_user_cpu_usage(db, session, new_user, run_workflow):
         (ResourceUnit.bytes_, 0, "0 Bytes"),
         (ResourceUnit.bytes_, 1024 * 35, "35 KiB"),
         (ResourceUnit.bytes_, 1024 * 200 + 512, "200.5 KiB"),
-        (ResourceUnit.bytes_, 1024 ** 2, "1 MiB"),
-        (ResourceUnit.bytes_, 1024 ** 2 + 1024 * 768, "1.75 MiB"),
-        (ResourceUnit.bytes_, 1024 ** 3 * 5 + 1024 ** 2 * 100, "5.1 GiB"),
-        (ResourceUnit.bytes_, 1024 ** 4 + 1024 ** 3 * 256, "1.25 TiB"),
+        (ResourceUnit.bytes_, 1024**2, "1 MiB"),
+        (ResourceUnit.bytes_, 1024**2 + 1024 * 768, "1.75 MiB"),
+        (ResourceUnit.bytes_, 1024**3 * 5 + 1024**2 * 100, "5.1 GiB"),
+        (ResourceUnit.bytes_, 1024**4 + 1024**3 * 256, "1.25 TiB"),
     ],
 )
 def test_human_readable_unit_values(unit, value, human_readable_string):
@@ -331,7 +455,10 @@ def test_should_cleanup_job(
     ],
 )
 def test_get_workflow_overload_priority(
-    run_workflow, running_workflows, REANA_MAX_CONCURRENT_BATCH_WORKFLOWS, priority,
+    run_workflow,
+    running_workflows,
+    REANA_MAX_CONCURRENT_BATCH_WORKFLOWS,
+    priority,
 ):
     """Test logic to determine workflow overload priority factor based on running workflows."""
     with mock.patch(
